
/* in mailcmd.c */
extern int do_list(int argc, char **argv);
extern int do_read(int argc, char **argv);
extern int do_send(int argc, char **argv);
extern int do_fax(int argc, char **argv);
extern int do_kill(int argc, char **argv);
extern int do_unkill(int argc, char **argv);

