
/* in command.c */
extern int do_status(int argc, char **argv);
extern int do_name(int argc, char **argv);
extern int do_help(int argc, char **argv);
extern int do_quit(int argc, char **argv);
extern int do_exit(int argc, char **argv);
extern int do_forward(int argc, char **argv); 
/* extern int do_forward(void); */
